/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Written (W) 1999-2007 Soeren Sonnenburg
 * Written (W) 1999-2007 Gunnar Raetsch
 * Written (W) 2007-2008 Christian Widmer
 * Copyright (C) 1999-2007 Fraunhofer Institute FIRST and Max-Planck-Society
 */

#ifndef _LABELS__H__
#define _LABELS__H__

#include "lib/common.h"
#include "lib/io.h"
#include "base/SGObject.h"

class CLabels : public CSGObject
{

#ifdef HAVE_BOOST_SERIALIZATION
  private:

    // serialization needs to split up in save/load because 
    // the serialization of pointers to natives (int* & friends) 
    // requires a workaround 
    friend class boost::serialization::access;
    template<class Archive>
    void save(Archive & ar, const unsigned int version) const
    {
    
      std::cout << "archiving Labels" << std::endl;

      ar & boost::serialization::base_object<CSGObject>(*this);

      ar & num_labels;
      for (int i=0; i < num_labels; ++i) 
      {
        ar & labels[i];
      }
  
      std::cout << "done with Labels" << std::endl;

    }
  
    template<class Archive>
    void load(Archive & ar, const unsigned int version)
    {
      
      std::cout << "archiving Labels" << std::endl;

      ar & boost::serialization::base_object<CSGObject>(*this);

      ar & num_labels;
            
      std::cout << "num_labels: " << num_labels << std::endl;

      if (num_labels > 0)
      {
  
        labels = new DREAL[num_labels];
        for (int i=0; i< num_labels; ++i) 
        {
          ar & labels[i];
        }
  
      }
  
      std::cout << "done with Labels" << std::endl;

    }
  
    BOOST_SERIALIZATION_SPLIT_MEMBER();


  public:

    virtual std::string toString() const
    {
      std::ostringstream s;

      boost::archive::text_oarchive oa(s);

      oa << *this;

      return s.str();
    }


    virtual void fromString(std::string str)
    {

      std::istringstream is(str);

      boost::archive::text_iarchive ia(is);

      ia >> *this;

    }



#endif //HAVE_BOOST_SERIALIZATION


	public:
		CLabels();
		CLabels(INT num_labels);
		CLabels(CHAR* fname);
		~CLabels();

		bool load(CHAR* fname);
		bool save(CHAR* fname);

		/// set/get the labels
		inline bool set_label(INT idx, DREAL label)
		{ 
			if (labels && idx<num_labels)
			{
				labels[idx]=label;
				return true;
			}
			else 
				return false;
		}

		/// set/get INT label
		inline bool set_int_label(INT idx, INT label)
		{ 
			if (labels && idx<num_labels)
			{
				labels[idx]= (DREAL) label;
				return true;
			}
			else 
				return false;
		}

		inline DREAL get_label(INT idx)
		{
			if (labels && idx<num_labels)
				return labels[idx];
			else
				return -1;
		}

		inline INT get_int_label(INT idx)
		{
			if (labels && idx<num_labels)
			{
				ASSERT(labels[idx]== ((DREAL) ((INT) labels[idx])));
				return ((INT) labels[idx]);
			}
			else
				return -1;
		}

		bool is_two_class_labeling();

		/// return number of classes (for multiclass)
		/// labels have to be zero based 0,1,...C missing
		/// labels are illegal
		INT get_num_classes();

		/// get label vector
		/// caller has to clean up
		DREAL* get_labels(INT &len) ;
		
		void get_labels(DREAL** labels, INT* len);

		/// set label vector
		void set_labels(DREAL* labels, INT len);

		/// get INT label vector
		/// caller has to clean up
		INT* get_int_labels(INT &len) ;

		/// set INT label vector
		/// caller has to clean up
		void set_int_labels(INT *labels, INT len) ;

		/// get number of labels
		inline INT get_num_labels() { return num_labels; }
	protected:
		INT num_labels;
		DREAL* labels;
};

#endif
